@@ -10,6 +10,7 @@ Spyder kernel for Jupyter
 
 # Standard library imports
 import os
+import os.path as osp
 
 # Third-party imports
 from ipykernel.datapub import publish_data
@@ -65,6 +66,14 @@ class SpyderKernel(IPythonKernel):
         self._pdb_obj = None
         self._pdb_step = None
 
+        kernel_config = self.config.get('IPKernelApp', None)
+        if kernel_config is not None:
+            cf = kernel_config['connection_file']
+            json_file = osp.basename(cf)
+            self._kernel_id = json_file.split('.json')[0]
+        else:
+            self._kernel_id = None
+
     @property
     def _pdb_frame(self):
         """Return current Pdb frame if there is any"""
@@ -137,7 +146,15 @@ class SpyderKernel(IPythonKernel):
         """Get the value of a variable"""
         ns = self._get_current_namespace()
         value = ns[name]
-        publish_data({'__spy_data__': value})
+        try:
+            publish_data({'__spy_data__': value})
+        except:
+            # * There is no need to inform users about
+            #   these errors.
+            # * value = None makes Spyder to ignore
+            #   petitions to display a value
+            value = None
+            publish_data({'__spy_data__': value})
 
     def set_value(self, name, value):
         """Set the value of a variable"""
@@ -192,6 +209,17 @@ class SpyderKernel(IPythonKernel):
         """Return info about pdb current frame"""
         return self._pdb_step
 
+    def publish_pdb_state(self):
+        """
+        Publish Variable Explorer state and Pdb step through
+        publish_data.
+        """
+        if self._pdb_obj:
+            state = dict(namespace_view = self.get_namespace_view(),
+                         var_properties = self.get_var_properties(),
+                         step = self._pdb_step)
+            publish_data({'__spy_pdb_state__': state})
+
     # --- For the Help plugin
     def is_defined(self, obj, force_import=False):
         """Return True if object is defined in current namespace"""
@@ -210,10 +238,33 @@ class SpyderKernel(IPythonKernel):
         if valid:
             return getsource(obj)
 
+    # --- Additional methods
     def set_cwd(self, dirname):
         """Set current working directory."""
         return os.chdir(dirname)
 
+    def get_cwd(self):
+        """Get current working directory."""
+        return os.getcwd()
+
+    def get_syspath(self):
+        """Return sys.path contents."""
+        import sys
+        return sys.path[:]
+
+    def get_env(self):
+        """Get environment variables."""
+        return os.environ.copy()
+
+    def close_all_mpl_figures(self):
+        """Close all Matplotlib figures."""
+        try:
+            import matplotlib.pyplot as plt
+            plt.close('all')
+            del plt
+        except:
+            pass
+
     # -- Private API ---------------------------------------------------
     # --- For the Variable Explorer
     def _get_current_namespace(self, with_magics=False):
@@ -277,7 +328,7 @@ class SpyderKernel(IPythonKernel):
         try:
             import numpy
             return isinstance(var, numpy.ndarray)
-        except ImportError:
+        except:
             return False
 
     def _is_image(self, var):
@@ -285,7 +336,7 @@ class SpyderKernel(IPythonKernel):
         try:
             from PIL import Image
             return isinstance(var, Image.Image)
-        except ImportError:
+        except:
             return False
 
     def _is_data_frame(self, var):
